==> nextai_vllm-0.0.1-cp310-cp310-manylinux1_x86_64.whl: vllm/entrypoints/openai/api_server.py <==
import argparse
import asyncio
import json
from contextlib import asynccontextmanager
import os
import importlib
import inspect

from aioprometheus import MetricsMiddleware
from aioprometheus.asgi.starlette import metrics
import fastapi
import uvicorn
import requests
import GPUtil
from http import HTTPStatus
from fastapi import Request, Depends, BackgroundTasks
from fastapi.exceptions import RequestValidationError, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from typing import Optional, List, Annotated
from fastapi.security import HTTPAuthorizationCredentials, HTTPBearer
from fastapi.responses import JSONResponse, StreamingResponse, Response
from sse_starlette.sse import EventSourceResponse
from asyncio import sleep
from json import dumps
import sh

from vllm.engine.arg_utils import AsyncEngineArgs
from vllm.engine.async_llm_engine import AsyncLLMEngine
from vllm.engine.metrics import add_global_metrics_labels
from vllm.entrypoints.openai.protocol import CompletionRequest, ChatCompletionRequest, ErrorResponse, AppSettings
from vllm.logger import init_logger, _LOGFILE
from vllm.entrypoints.openai.serving_chat import OpenAIServingChat
from vllm.entrypoints.openai.serving_completion import OpenAIServingCompletion


TIMEOUT_KEEP_ALIVE = 5  # seconds

openai_serving_chat: OpenAIServingChat = None
openai_serving_completion: OpenAIServingCompletion = None
logger = init_logger(__name__)

get_bearer_token = HTTPBearer(auto_error=False)


@asynccontextmanager
async def lifespan(app: fastapi.FastAPI):

    async def _force_log():
        while True:
            await asyncio.sleep(10)
            await engine.do_log_stats()

    if not engine_args.disable_log_stats:
        asyncio.create_task(_force_log())

    yield


app = fastapi.FastAPI(lifespan=lifespan)
app_settings = AppSettings()

# Added API key check
async def check_api_key(
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
) -> str:
    if auth is None:
        raise HTTPException(
            status_code=401,
            detail={
                "error": {
                    "message": "Invalid API Key",
                    "type": "invalid_request_error",
                    "param": None,
                    "code": "invalid_api_key",
                }
            },
        )
    if app_settings.cluster_id:
        token = auth.credentials
        request = requests.get(url='{}/public/clusters/{}/ping'.format(app_settings.auth_server, app_settings.cluster_id), headers={
            "x-api-key": auth.credentials
        })
        if request.status_code == 200:
            return token
        else:
            raise HTTPException(
                status_code=403,
                detail={
                    "error": {
                        "message": "Invalid API Key",
                        "type": "invalid_request_error",
                        "param": None,
                        "code": "invalid_api_key",
                    }
                },
            )
    else:
        token = auth.credentials
        request = requests.get(url='{}/public/ping'.format(app_settings.auth_server), headers={
            "x-api-key": auth.credentials
        })
        if request.status_code == 200:
            return token
        else:
            raise HTTPException(
                status_code=403,
                detail={
                    "error": {
                        "message": "Invalid API Key",
                        "type": "invalid_request_error",
                        "param": None,
                        "code": "invalid_api_key",
                    }
                },
            )


def parse_args():
    parser = argparse.ArgumentParser(
        description="NextAI-vLLM OpenAI-Compatible RESTful API server.")
    parser.add_argument("--host", type=str, default=None, help="host name")
    parser.add_argument("--port", type=int, default=8000, help="port number")
    parser.add_argument("--allow-credentials",
                        action="store_true",
                        help="allow credentials")
    parser.add_argument("--allowed-origins",
                        type=json.loads,
                        default=["*"],
                        help="allowed origins")
    parser.add_argument("--allowed-methods",
                        type=json.loads,
                        default=["*"],
                        help="allowed methods")
    parser.add_argument("--allowed-headers",
                        type=json.loads,
                        default=["*"],
                        help="allowed headers")
    parser.add_argument("--cluster",
                        type=str,
                        help="id of the cluster")
    parser.add_argument("--api-key",
                        type=str,
                        default=None,
                        help="If provided, the server will require this key to be presented in the header.")
    parser.add_argument("--served-model-name",
                        type=str,
                        default=None,
                        help="The model name used in the API. If not "
                        "specified, the model name will be the same as "
                        "the huggingface name.")
    parser.add_argument("--chat-template",
                        type=str,
                        default=None,
                        help="The file path to the chat template, "
                        "or the template in single-line form "
                        "for the specified model")
    parser.add_argument("--response-role",
                        type=str,
                        default="assistant",
                        help="The role name to return if "
                        "`request.add_generation_prompt=true`.")
    parser.add_argument("--ssl-keyfile",
                        type=str,
                        default=None,
                        help="The file path to the SSL key file")
    parser.add_argument("--ssl-certfile",
                        type=str,
                        default=None,
                        help="The file path to the SSL cert file")
    parser.add_argument("--auth-server",
                        type=str,
                        help="token authentication server",
                        default='https://api.nextai.co.in')
    parser.add_argument(
        "--root-path",
        type=str,
        default=None,
        help="FastAPI root_path when app is behind a path based routing proxy")
    parser.add_argument(
        "--middleware",
        type=str,
        action="append",
        default=[],
        help="Additional ASGI middleware to apply to the app. "
        "We accept multiple --middleware arguments. "
        "The value should be an import path. "
        "If a function is provided, vLLM will add it to the server using @app.middleware('http'). "
        "If a class is provided, vLLM will add it to the server using app.add_middleware(). "
    )

    parser = AsyncEngineArgs.add_cli_args(parser)
    return parser.parse_args()


app.add_middleware(MetricsMiddleware)  # Trace HTTP server metrics
app.add_route("/metrics", metrics)  # Exposes HTTP metrics


@app.exception_handler(RequestValidationError)
async def validation_exception_handler(_, exc):
    err = openai_serving_chat.create_error_response(message=str(exc))
    return JSONResponse(err.model_dump(), status_code=HTTPStatus.BAD_REQUEST)


@app.get("/health")
async def health() -> Response:
    """Health check."""
    return Response(status_code=200)


@app.get("/v1/models")
async def show_available_models(token: Annotated[str, Depends(check_api_key)], background_tasks: BackgroundTasks):
    models = await openai_serving_chat.show_available_models()
    return JSONResponse(content=models.model_dump())


@app.post("/v1/chat/completions")
async def create_chat_completion(request: ChatCompletionRequest,
                                 raw_request: Request, token: Annotated[str, Depends(check_api_key)], background_tasks: BackgroundTasks):
    generator = await openai_serving_chat.create_chat_completion(
        request, raw_request, app_settings=app_settings, token=token, background_tasks=background_tasks)
    if isinstance(generator, ErrorResponse):
        return JSONResponse(content=generator.model_dump(),
                            status_code=generator.code)
    if request.stream:
        return StreamingResponse(content=generator,
                                 media_type="text/event-stream")
    else:
        return JSONResponse(content=generator.model_dump())


@app.post("/v1/completions")
async def create_completion(request: CompletionRequest, raw_request: Request, token: Annotated[str, Depends(check_api_key)], background_tasks: BackgroundTasks):
    generator = await openai_serving_completion.create_completion(
        request, raw_request, settings=app_settings, token=token, background_tasks=background_tasks)
    if isinstance(generator, ErrorResponse):
        return JSONResponse(content=generator.model_dump(),
                            status_code=generator.code)
    if request.stream:
        return StreamingResponse(content=generator,
                                 media_type="text/event-stream")
    else:
        return JSONResponse(content=generator.model_dump())
    
async def follow(request):
    '''generator function that yields new lines in a file
    '''
    file = open(_LOGFILE, "r")
    # seek the end of the file
    file.seek(0, os.SEEK_END)
    
    # start infinite loop
    while True:
        if await request.is_disconnected():
            print("Client disconnected !")
            break
        # read last line of file
        line = file.readline()
        # sleep if file hasn't been updated
        if not line:
            await sleep(0.5)
            continue
        yield line

@app.get("/log_stream")
async def log_stream(request: Request):
    return EventSourceResponse(follow(request))


@app.get("/worker_gpu_status_stream")
async def api_worker_gpu_status_stream(request: Request):
    async def gpu_status_stream():
        while True:
            response = []
            for gpu in GPUtil.getGPUs():
                response.append({
                    "id": gpu.id,
                    "uuid": gpu.uuid,
                    "load": gpu.load,
                    "memoryUtil": gpu.memoryUtil,
                    "memoryTotal": gpu.memoryTotal,
                    "memoryUsed": gpu.memoryUsed,
                    "memoryFree": gpu.memoryFree,
                    "driver": gpu.driver,
                    "name": gpu.name,
                    "serial": gpu.serial,
                    "display_mode": gpu.display_mode,
                    "display_active": gpu.display_active,
                    "temperature": gpu.temperature
                })
            yield dumps(response)
            await sleep(3)
    return EventSourceResponse(gpu_status_stream())


@app.get("/worker_status_stream")
async def api_worker_status_stream(request: Request):
    async def worker_status_stream():
        while True:
            yield "OK"
            await sleep(3)
    return EventSourceResponse(worker_status_stream())

if __name__ == "__main__":
    args = parse_args()

    app.add_middleware(
        CORSMiddleware,
        allow_origins=args.allowed_origins,
        allow_credentials=args.allow_credentials,
        allow_methods=args.allowed_methods,
        allow_headers=args.allowed_headers,
    )

    app_settings.cluster_id = args.cluster if args.cluster is not None else os.environ.get("NEXTAI_CLUSTER_ID", None)
    app_settings.auth_server = args.auth_server if args.auth_server is not None else 'https://api.nextai.co.in'

    for middleware in args.middleware:
        module_path, object_name = middleware.rsplit(".", 1)
        imported = getattr(importlib.import_module(module_path), object_name)
        if inspect.isclass(imported):
            app.add_middleware(imported)
        elif inspect.iscoroutinefunction(imported):
            app.middleware("http")(imported)
        else:
            raise ValueError(
                f"Invalid middleware {middleware}. Must be a function or a class."
            )

    logger.info(f"args: {args}")

    if args.served_model_name is not None:
        served_model = args.served_model_name
    else:
        served_model = args.model

    engine_args = AsyncEngineArgs.from_cli_args(args)
    engine = AsyncLLMEngine.from_engine_args(engine_args)
    openai_serving_chat = OpenAIServingChat(engine, served_model,
                                            args.response_role,
                                            args.chat_template)
    openai_serving_completion = OpenAIServingCompletion(engine, served_model)

    # Register labels for metrics
    add_global_metrics_labels(model_name=engine_args.model)

    app.root_path = args.root_path
    uvicorn.run(app,
                host=args.host,
                port=args.port,
                log_level="info",
                timeout_keep_alive=TIMEOUT_KEEP_ALIVE,
                ssl_keyfile=args.ssl_keyfile,
                ssl_certfile=args.ssl_certfile)

==> nextai_vllm-0.0.7-cp310-cp310-manylinux1_x86_64.whl: vllm/entrypoints/openai/api_server.py <==
import asyncio
import importlib
import inspect
import os
from contextlib import asynccontextmanager
from http import HTTPStatus

import fastapi
import uvicorn
import requests
import GPUtil
from http import HTTPStatus
from fastapi import Request, Depends, BackgroundTasks
from fastapi.exceptions import RequestValidationError, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from typing import Optional, List, Annotated
from fastapi.security import HTTPAuthorizationCredentials, HTTPBearer
from fastapi.responses import JSONResponse, StreamingResponse, Response
from sse_starlette.sse import EventSourceResponse
from asyncio import sleep
from json import dumps
import json
import sh
from prometheus_client import make_asgi_app
import argparse

import vllm
from vllm.engine.arg_utils import AsyncEngineArgs
from vllm.engine.async_llm_engine import AsyncLLMEngine
from vllm.entrypoints.openai.protocol import CompletionRequest, ChatCompletionRequest, ErrorResponse, AppSettings
from vllm.logger import init_logger, _LOGFILE
from vllm.entrypoints.openai.serving_chat import OpenAIServingChat
from vllm.entrypoints.openai.serving_completion import OpenAIServingCompletion
from vllm.logger import init_logger
from vllm.usage.usage_lib import UsageContext
from vllm.entrypoints.openai.cli_args import LoRAParserAction


TIMEOUT_KEEP_ALIVE = 5  # seconds

openai_serving_chat: OpenAIServingChat = None
openai_serving_completion: OpenAIServingCompletion = None
logger = init_logger(__name__)

get_bearer_token = HTTPBearer(auto_error=False)


@asynccontextmanager
async def lifespan(app: fastapi.FastAPI):

    async def _force_log():
        while True:
            await asyncio.sleep(10)
            await engine.do_log_stats()

    if not engine_args.disable_log_stats:
        asyncio.create_task(_force_log())

    yield


app = fastapi.FastAPI(lifespan=lifespan)
app_settings = AppSettings()

# Added API key check
async def check_api_key(
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
) -> str:
    if auth is None:
        raise HTTPException(
            status_code=401,
            detail={
                "error": {
                    "message": "Invalid API Key",
                    "type": "invalid_request_error",
                    "param": None,
                    "code": "invalid_api_key",
                }
            },
        )
    if app_settings.cluster_id:
        token = auth.credentials
        request = requests.get(url='{}/public/clusters/{}/ping'.format(app_settings.auth_server, app_settings.cluster_id), headers={
            "x-api-key": auth.credentials
        })
        if request.status_code == 200:
            return token
        else:
            raise HTTPException(
                status_code=403,
                detail={
                    "error": {
                        "message": "Invalid API Key",
                        "type": "invalid_request_error",
                        "param": None,
                        "code": "invalid_api_key",
                    }
                },
            )
    else:
        token = auth.credentials
        request = requests.get(url='{}/public/ping'.format(app_settings.auth_server), headers={
            "x-api-key": auth.credentials
        })
        if request.status_code == 200:
            return token
        else:
            raise HTTPException(
                status_code=403,
                detail={
                    "error": {
                        "message": "Invalid API Key",
                        "type": "invalid_request_error",
                        "param": None,
                        "code": "invalid_api_key",
                    }
                },
            )


def parse_args():
    parser = argparse.ArgumentParser(
        description="NextAI-vLLM OpenAI-Compatible RESTful API server.")
    parser.add_argument("--host", type=str, default=None, help="host name")
    parser.add_argument("--port", type=int, default=8000, help="port number")
    parser.add_argument("--allow-credentials",
                        action="store_true",
                        help="allow credentials")
    parser.add_argument("--allowed-origins",
                        type=json.loads,
                        default=["*"],
                        help="allowed origins")
    parser.add_argument("--allowed-methods",
                        type=json.loads,
                        default=["*"],
                        help="allowed methods")
    parser.add_argument("--allowed-headers",
                        type=json.loads,
                        default=["*"],
                        help="allowed headers")
    parser.add_argument("--cluster",
                        type=str,
                        help="id of the cluster")
    parser.add_argument("--api-key",
                        type=str,
                        default=None,
                        help="If provided, the server will require this key to be presented in the header.")
    parser.add_argument("--served-model-name",
                        type=str,
                        default=None,
                        help="The model name used in the API. If not "
                        "specified, the model name will be the same as "
                        "the huggingface name.")
    parser.add_argument("--chat-template",
                        type=str,
                        default=None,
                        help="The file path to the chat template, "
                        "or the template in single-line form "
                        "for the specified model")
    parser.add_argument("--response-role",
                        type=str,
                        default="assistant",
                        help="The role name to return if "
                        "`request.add_generation_prompt=true`.")
    parser.add_argument("--ssl-keyfile",
                        type=str,
                        default=None,
                        help="The file path to the SSL key file")
    parser.add_argument("--ssl-certfile",
                        type=str,
                        default=None,
                        help="The file path to the SSL cert file")
    parser.add_argument("--auth-server",
                        type=str,
                        help="token authentication server",
                        default='https://api.nextai.co.in')
    parser.add_argument(
        "--lora-modules",
        type=str,
        default=None,
        nargs='+',
        action=LoRAParserAction,
        help="LoRA module configurations in the format name=path. "
        "Multiple modules can be specified.")
    parser.add_argument(
        "--root-path",
        type=str,
        default=None,
        help="FastAPI root_path when app is behind a path based routing proxy")
    parser.add_argument(
        "--middleware",
        type=str,
        action="append",
        default=[],
        help="Additional ASGI middleware to apply to the app. "
        "We accept multiple --middleware arguments. "
        "The value should be an import path. "
        "If a function is provided, vLLM will add it to the server using @app.middleware('http'). "
        "If a class is provided, vLLM will add it to the server using app.add_middleware(). "
    )

    parser = AsyncEngineArgs.add_cli_args(parser)
    return parser.parse_args()


# Add prometheus asgi middleware to route /metrics requests
metrics_app = make_asgi_app()
app.mount("/metrics", metrics_app)


@app.exception_handler(RequestValidationError)
async def validation_exception_handler(_, exc):
    err = openai_serving_chat.create_error_response(message=str(exc))
    return JSONResponse(err.model_dump(), status_code=HTTPStatus.BAD_REQUEST)


@app.get("/health")
async def health() -> Response:
    """Health check."""
    await openai_serving_chat.engine.check_health()
    return Response(status_code=200)


@app.get("/v1/models")
async def show_available_models(token: Annotated[str, Depends(check_api_key)], background_tasks: BackgroundTasks):
    models = await openai_serving_chat.show_available_models()
    return JSONResponse(content=models.model_dump())


@app.get("/version")
async def show_version():
    ver = {"version": vllm.__version__}
    return JSONResponse(content=ver)


@app.post("/v1/chat/completions")
async def create_chat_completion(request: ChatCompletionRequest,
                                 raw_request: Request, token: Annotated[str, Depends(check_api_key)], background_tasks: BackgroundTasks):
    generator = await openai_serving_chat.create_chat_completion(
        request, raw_request, app_settings=app_settings, token=token, background_tasks=background_tasks)
    if isinstance(generator, ErrorResponse):
        return JSONResponse(content=generator.model_dump(),
                            status_code=generator.code)
    if request.stream:
        return StreamingResponse(content=generator,
                                 media_type="text/event-stream")
    else:
        return JSONResponse(content=generator.model_dump())


@app.post("/v1/completions")
async def create_completion(request: CompletionRequest, raw_request: Request, token: Annotated[str, Depends(check_api_key)], background_tasks: BackgroundTasks):
    generator = await openai_serving_completion.create_completion(
        request, raw_request, settings=app_settings, token=token, background_tasks=background_tasks)
    if isinstance(generator, ErrorResponse):
        return JSONResponse(content=generator.model_dump(),
                            status_code=generator.code)
    if request.stream:
        return StreamingResponse(content=generator,
                                 media_type="text/event-stream")
    else:
        return JSONResponse(content=generator.model_dump())
    
async def follow(request):
    '''generator function that yields new lines in a file
    '''
    file = open(_LOGFILE, "r")
    # seek the end of the file
    file.seek(0, os.SEEK_END)
    
    # start infinite loop
    while True:
        if await request.is_disconnected():
            print("Client disconnected !")
            break
        # read last line of file
        line = file.readline()
        # sleep if file hasn't been updated
        if not line:
            await sleep(0.5)
            continue
        yield line

@app.get("/log_stream")
async def log_stream(request: Request):
    return EventSourceResponse(follow(request))


@app.get("/worker_gpu_status_stream")
async def api_worker_gpu_status_stream(request: Request):
    async def gpu_status_stream():
        while True:
            response = []
            for gpu in GPUtil.getGPUs():
                response.append({
                    "id": gpu.id,
                    "uuid": gpu.uuid,
                    "load": gpu.load,
                    "memoryUtil": gpu.memoryUtil,
                    "memoryTotal": gpu.memoryTotal,
                    "memoryUsed": gpu.memoryUsed,
                    "memoryFree": gpu.memoryFree,
                    "driver": gpu.driver,
                    "name": gpu.name,
                    "serial": gpu.serial,
                    "display_mode": gpu.display_mode,
                    "display_active": gpu.display_active,
                    "temperature": gpu.temperature
                })
            yield dumps(response)
            await sleep(3)
    return EventSourceResponse(gpu_status_stream())


@app.get("/worker_status_stream")
async def api_worker_status_stream(request: Request):
    async def worker_status_stream():
        while True:
            yield "OK"
            await sleep(3)
    return EventSourceResponse(worker_status_stream())

if __name__ == "__main__":
    args = parse_args()

    app.add_middleware(
        CORSMiddleware,
        allow_origins=args.allowed_origins,
        allow_credentials=args.allow_credentials,
        allow_methods=args.allowed_methods,
        allow_headers=args.allowed_headers,
    )

    app_settings.cluster_id = args.cluster if args.cluster is not None else os.environ.get("NEXTAI_CLUSTER_ID", None)
    app_settings.auth_server = args.auth_server if args.auth_server is not None else 'https://api.nextai.co.in'

    for middleware in args.middleware:
        module_path, object_name = middleware.rsplit(".", 1)
        imported = getattr(importlib.import_module(module_path), object_name)
        if inspect.isclass(imported):
            app.add_middleware(imported)
        elif inspect.iscoroutinefunction(imported):
            app.middleware("http")(imported)
        else:
            raise ValueError(f"Invalid middleware {middleware}. "
                             f"Must be a function or a class.")

    logger.info(f"vLLM API server version {vllm.__version__}")
    logger.info(f"args: {args}")

    if args.served_model_name is not None:
        served_model = args.served_model_name
    else:
        served_model = args.model
    engine_args = AsyncEngineArgs.from_cli_args(args)
    engine = AsyncLLMEngine.from_engine_args(
        engine_args, usage_context=UsageContext.OPENAI_API_SERVER)
    openai_serving_chat = OpenAIServingChat(engine, served_model,
                                            args.response_role,
                                            args.lora_modules,
                                            args.chat_template,
                                            app_settings=app_settings)
    openai_serving_completion = OpenAIServingCompletion(
        engine, served_model, args.lora_modules, app_settings=app_settings)

    app.root_path = args.root_path
    uvicorn.run(app,
                host=args.host,
                port=args.port,
                log_level="info",
                timeout_keep_alive=TIMEOUT_KEEP_ALIVE,
                ssl_keyfile=args.ssl_keyfile,
                ssl_certfile=args.ssl_certfile)
